<script setup lang="ts">
import type { Recordable } from '@vben/types';

import { onMounted } from 'vue';

import { Page, useVbenDrawer, type VbenFormProps } from '@vben/common-ui';
import { $t } from '@vben/locales';
import { getVxePopupContainer } from '@vben/utils';

import { Modal, Popconfirm, Space } from 'ant-design-vue';

import { useVbenVxeGrid, vxeCheckboxChecked, type VxeGridProps, vxeSortEvent } from '#/adapter/vxe-table';
import { listEqu } from '#/api/eims/equ';
import { delEquStatu, equStatuExport, listEquStatu } from '#/api/eims/equ-statu';
import { commonDownloadExcel } from '#/utils/file/download';

import { columns, querySchema } from './data';
import equStatuDrawer from './equ-statu-drawer.vue';
import statuPreviewDrawer from './statu-preview-drawer.vue';

// 从设备明细打开页面 1.不需要设备筛选 2.只查询当前设备数据
interface Props {
  equDetailFlag?: boolean;
  equId?: string;
}

const props = withDefaults(defineProps<Props>(), { equDetailFlag: false, equId: undefined });

const formOptions: VbenFormProps = {
  commonConfig: {
    labelWidth: 80,
    componentProps: {
      allowClear: true
    }
  },
  schema: querySchema(),
  wrapperClass: 'grid-cols-1 md:grid-cols-2 lg:grid-cols-3 xl:grid-cols-4',
  // 日期选择格式化
  fieldMappingTime: [['changeDate', ['params[beginTime]', 'params[endTime]'], ['YYYY-MM-DD 00:00:00', 'YYYY-MM-DD 23:59:59']]]
};

const gridOptions: VxeGridProps = {
  checkboxConfig: {
    // 高亮
    highlight: true,
    // 翻页时保留选中状态
    reserve: true
    // 点击行选中
    // trigger: 'row'
  },
  columns,
  height: 'auto',
  keepSource: true,
  pagerConfig: {},
  toolbarConfig: {
    enabled: !props.equDetailFlag
  },
  proxyConfig: {
    ajax: {
      query: async ({ page }, formValues = {}) => {
        // 如果传入了equId则只查询当前id数据
        if (props.equDetailFlag && props.equId) {
          const queryEqu = { equId: props.equId };
          Object.assign(formValues, queryEqu);
        }

        return await listEquStatu({
          pageNum: page.currentPage,
          pageSize: page.pageSize,
          ...formValues
        });
      }
    }
  },
  rowConfig: {
    isHover: true,
    keyField: 'equStatuId'
  },
  sortConfig: {
    // 远程排序
    remote: true,
    // 支持多字段排序 默认关闭
    multiple: true,
  },
  id: 'eims-equ-statu-index'
};

const [BasicTable, tableApi] = useVbenVxeGrid({
  formOptions,
  gridOptions,
  gridEvents: {
    sortChange: (sortParams) => vxeSortEvent(tableApi, sortParams),
  },
});

const [EquStatuDrawer, equStatuDrawerApi] = useVbenDrawer({
  connectedComponent: equStatuDrawer
});

const [StatuPreviewDrawer, statuPreviewDrawerApi] = useVbenDrawer({
  connectedComponent: statuPreviewDrawer
});

/**
 * 预览
 * @param record
 */
function handlePreview(record: Recordable<any>) {
  statuPreviewDrawerApi.setData({ record });
  statuPreviewDrawerApi.open();
}

onMounted(async () => {
  await setupEquSelect();
});

async function setupEquSelect() {
  const equPageResult = await listEqu({
    pageNum: 1,
    pageSize: 1000
  });
  if (!equPageResult || equPageResult.rows.length < 0) {
    return false;
  }
  // 使用map来跟踪已经遇到的equId，使用filter来过滤掉重复的元素。
  const uniqueItems = equPageResult.rows.filter((item, index, self) => index === self.findIndex((tm) => tm.equId === item.equId));

  const options = uniqueItems.map((item) => ({
    label: item.equName || item.equName,
    value: item.equId
  }));
  // 筛选
  const filterOption = (input: string, option: any) => {
    return option.label.toLowerCase().includes(input.toLowerCase());
  };

  const placeholder = options.length > 0 ? '请选择' : '暂无设备记录';
  // 更新selectOptions
  tableApi.formApi.updateSchema([
    {
      componentProps: {
        options,
        placeholder,
        filterOption
      },
      dependencies: {
        show: () => !props.equDetailFlag,
        triggerFields: ['']
      },
      fieldName: 'equId'
    }
  ]);
}

function handleAdd() {
  // 设备详情页打开时 只能新增指定设备id数据
  if (props.equDetailFlag && props.equId) {
    equStatuDrawerApi.setData({ equId: props.equId });
  } else {
    equStatuDrawerApi.setData({});
  }

  equStatuDrawerApi.open();
}

async function handleEdit(record: Recordable<any>) {
  equStatuDrawerApi.setData({ id: record.equStatuId });
  equStatuDrawerApi.open();
}

async function handleDelete(row: Recordable<any>) {
  await delEquStatu(row.equStatuId);
  await tableApi.query();
}

function handleMultiDelete() {
  const rows = tableApi.grid.getCheckboxRecords();
  const ids = rows.map((row: any) => row.equStatuId);
  Modal.confirm({
    title: '提示',
    okType: 'danger',
    content: `确认删除选中的${ids.length}条记录吗？`,
    onOk: async () => {
      await delEquStatu(ids);
      await tableApi.query();
    }
  });
}

function handleDownloadExcel() {
  commonDownloadExcel(equStatuExport, '设备状态变更记录', tableApi.formApi.form.values, {
    fieldMappingTime: formOptions.fieldMappingTime
  });
}
</script>

<template>
  <Page :auto-content-height="true">
    <div class="flex h-full gap-[8px]">
      <BasicTable class="flex-1 overflow-hidden" table-title="试产列表">
        <template #toolbar-tools>
          <Space>
            <a-button v-access:code="['eims:equStatu:export']" @click="handleDownloadExcel">
              {{ $t('pages.common.export') }}
            </a-button>
            <a-button
              :disabled="!vxeCheckboxChecked(tableApi)"
              danger
              type="primary"
              v-access:code="['eims:equStatu:remove']"
              @click="handleMultiDelete"
            >
              {{ $t('pages.common.delete') }}
            </a-button>
            <a-button type="primary" v-access:code="['eims:equStatu:add']" @click="handleAdd">
              {{ $t('pages.common.add') }}
            </a-button>
          </Space>
        </template>

        <template #equName="{ row }">
          <Space>
            <span>{{ row.equName }}</span>
          </Space>
        </template>

        <template #action="{ row }">
          <Space>
            <ghost-button v-access:code="['eims:equStatu:edit']" @click.stop="handleEdit(row)">
              {{ $t('pages.common.edit') }}
            </ghost-button>
            <ghost-button class="btn-success" v-access:code="['eims:equStatu:list']" @click.stop="handlePreview(row)">
              {{ $t('pages.common.preview') }}
            </ghost-button>
            <Popconfirm :get-popup-container="getVxePopupContainer" placement="left" title="确认删除？" @confirm="handleDelete(row)">
              <ghost-button danger v-access:code="['eims:equStatu:remove']" @click.stop="">
                {{ $t('pages.common.delete') }}
              </ghost-button>
            </Popconfirm>
          </Space>
        </template>
      </BasicTable>
    </div>
    <EquStatuDrawer @reload="tableApi.query()" />
    <StatuPreviewDrawer />
  </Page>
</template>
